package org.example;
import org.graphstream.algorithm.generator.Generator;
import org.graphstream.algorithm.generator.BarabasiAlbertGenerator;
import org.graphstream.algorithm.generator.RandomGenerator;
import org.graphstream.graph.Graph;
import org.graphstream.graph.implementations.DefaultGraph;
import org.graphstream.graph.implementations.SingleGraph;
import org.graphstream.stream.file.FileSource;
import org.graphstream.stream.file.FileSourceEdge;
import org.graphstream.stream.file.FileSourceFactory;
import org.graphstream.algorithm.Toolkit;
import org.graphstream.graph.*;
import org.graphstream.graph.BreadthFirstIterator;

import java.io.*;
import java.time.Clock;
import java.util.*;

import static org.graphstream.algorithm.Toolkit.*;

/**
 * Hello world!
 *
 */
public class App 
{
    public static void mesureBase(Graph graphe){
        System.out.println("le nombre de noeuds du graphe est : "+ graphe.getNodeCount());
        System.out.println("le nombre de liens du graphe est : "+graphe.getEdgeCount());
        System.out.println("le degre moyen du graphe est : "+averageDegree(graphe));
        System.out.println("la coefficient de clustering est : "+averageClusteringCoefficient(graphe));
        System.out.println("est ce que le graphe est connexe : "+isConnected(graphe));
    }
    public static void distributionDegre(Graph graphe){
        int[] dd = Toolkit.degreeDistribution(graphe);
        try {
            PrintWriter fichier = new PrintWriter(new FileWriter("distributionDegre.dat"));
            fichier.write("La probabilité qu'un nœud choisi au hasard ait degré k : ");
            fichier.println();
            for (int k = 0; k < dd.length; k++) {
                // degreProba[i]/nbNoeud  : represente la probabilité  qu’un noeud choisi au hasard ait degré i
                fichier.write(k + "   " + (double) dd[k] / graphe.getNodeCount());
                fichier.println();
            }
            fichier.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    public static void distributionDistances(Graph graphe) throws IOException {
        HashMap<Integer,Integer> distr = new HashMap<>();
        int nb=0;
        for(int i = 0 ; i < 1000 ; i++){
            Node depart = randomNode(graphe);
            BreadthFirstIterator it = new BreadthFirstIterator(depart);
            while(it.hasNext()){
                Node next = it.next();
                int dist = it.getDepthOf(next);
                int num;
                if(distr.get(dist)!=null) num = distr.get(dist); else num = 0;
                distr.put(dist,num+1);
                nb++;
            }
        }
        File distDistances = new File("distributionDistances.dat");
        if (!distDistances.exists()) {
            distDistances.createNewFile();
        }
        FileWriter f = new FileWriter(distDistances.getAbsoluteFile());
        BufferedWriter b = new BufferedWriter(f);
        for (int distance: distr.keySet()){
            b.write(String.format(Locale.US, "%6d%20.8f%n", distance, (double)distr.get(distance) / nb));
        }
        b.close();
    }
    public static void reseauAleatoire(double degreMoyen, int nbNoeud){
        System.out.println("Réseau avec la méthode aleatoire : ");
        Graph graphe = new SingleGraph("Random");
        Generator gen = new RandomGenerator(degreMoyen);
        gen.addSink(graphe);
        gen.begin();
        for (int i = 0; i < nbNoeud; i++)
            gen.nextEvents();
        gen.end();

        System.out.println("le nombre de noeuds du graphe est : "+ graphe.getNodeCount());
        System.out.println("le nombre de liens du graphe est : "+graphe.getEdgeCount());
        System.out.println("le degre moyen du graphe est : "+averageDegree(graphe));
        System.out.println("la coefficient de clustering est : "+averageClusteringCoefficient(graphe));
        System.out.println("est ce que le graphe est connexe : "+isConnected(graphe));
        double somme = 0;
        int nbr=0;
        for(int i = 0 ; i < 1000 ; i++){
            Node depart = randomNode(graphe);
            BreadthFirstIterator it = new BreadthFirstIterator(depart);
            while(it.hasNext()){
                Node next = it.next();
                somme += it.getDepthOf(next);
                nbr++;
            }
        }
        double moyenne = somme/nbr;
        System.out.println("la distance moyenne : "+moyenne);
    }
    public static void reseauBarbasi(float degreMoyen, int nbNoeud){
        System.out.println("Réseau avec la méthode d'attachement préférentiel (Barabasi-Albert)  : ");
        Graph graphe = new SingleGraph("Barabàsi-Albert");
        // Between 1 and 3 new links per node added.
        Generator gen = new BarabasiAlbertGenerator((int)(degreMoyen));
        // Generate 100 nodes:
        gen.addSink(graphe);
        gen.begin();
        for(int i=0; i<nbNoeud; i++) {
            gen.nextEvents();
        }
        gen.end();

        System.out.println("le nombre de noeuds du graphe est : "+ graphe.getNodeCount());
        System.out.println("le nombre de liens du graphe est : "+graphe.getEdgeCount());
        System.out.println("le degre moyen du graphe est : "+averageDegree(graphe));
        System.out.println("la coefficient de clustering est : "+averageClusteringCoefficient(graphe));
        System.out.println("est ce que le graphe est connexe : "+isConnected(graphe));
        double somme = 0;
        int nbr=0;
        for(int i = 0 ; i < 1000 ; i++){
            Node depart = randomNode(graphe);
            BreadthFirstIterator it = new BreadthFirstIterator(depart);
            while(it.hasNext()){
                Node next = it.next();
                somme += it.getDepthOf(next);
                nbr++;
            }
        }
        double moyenne = somme/nbr;
        System.out.println("la distance moyenne : "+moyenne);
    }
    public static void main(String ... args) throws IOException {
        String filePath = "com-dblp.ungraph.txt";
        Graph g = new DefaultGraph("g");
        FileSource fs = new FileSourceEdge();
        fs.addSink(g);
        System.setProperty("org.graphstream.ui", "swing");
        try {
            fs.readAll(filePath);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            fs.removeSink(g);
        }
        mesureBase(g);
        distributionDegre(g);
        //distributionDistances(g);
        //reseauAleatoire( 6.622,317080);
        //reseauBarbasi((float) 6.622,317080);


    }
}
