package org.example;

import org.graphstream.algorithm.Toolkit;
import org.graphstream.algorithm.generator.Generator;
import org.graphstream.algorithm.generator.RandomGenerator;
import org.graphstream.graph.Edge;
import org.graphstream.graph.Graph;
import org.graphstream.graph.Node;
import org.graphstream.graph.implementations.DefaultGraph;
import org.graphstream.graph.implementations.SingleGraph;
import org.graphstream.stream.file.FileSource;
import org.graphstream.stream.file.FileSourceEdge;

import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.OptionalDouble;
import java.util.Random;

import static org.graphstream.algorithm.Toolkit.averageDegree;

public class propagation {
    public static double seuilEpidemiqueReseauAleatoire(double degreMoyen, int nbNoeud){
        System.out.println("Réseau aleatoire :");
        Graph graphe = new SingleGraph("Random");
        Generator gen = new RandomGenerator(degreMoyen);
        gen.addSink(graphe);
        gen.begin();
        for (int i = 0; i < nbNoeud; i++)
            gen.nextEvents();
        gen.end();
        double seuilEpidemique = 1 / (degreMoyen + 1);
        System.out.println("le seuil epidemique est : " + seuilEpidemique);
        return seuilEpidemique;
    }
    public static double seuilEpidemique(Graph graph){
        System.out.println("Notre réseau :");
        double degreMoyen = (float) averageDegree(graph);
        System.out.println("le degre moyen du graphe est : "+ degreMoyen);
        int[] dd = Toolkit.degreeDistribution(graph);
        OptionalDouble distributionDegre = Arrays.stream(dd).average();
        System.out.println("Distribution de degré : " + distributionDegre);
        double distribution = distributionDegre.getAsDouble();
        double seuilEpidemique = degreMoyen / distribution;
        System.out.println("le seuil epidemique est : " + seuilEpidemique);
        return seuilEpidemique;

    }
    public static int estInfecte(Node node, int nbInfection){
        double proba = Math.random();
        double envoieMail = 1.0/7.0;
        if(proba < envoieMail  && !node.hasAttribute("immunise")){
            if(!node.hasAttribute("infecte")){
                node.setAttribute("infecte", true);
                nbInfection++;
            }
        }
        return nbInfection;
    }

    public static int antiVerus(Node node, int nbInfection){
        double proba = Math.random();
        double antiVirus = 1.0/14.0;
        if(proba < antiVirus){
            if(node.hasAttribute("infecte")){
                node.removeAttribute("infecte");
                nbInfection--;
            }
        }
        return nbInfection;
    }

    public static void propagationScenario1(Graph graph) {
        Node s = graph.getNode(0);
        s.setAttribute("infecte", true);
        int nbInfect = 1;
        for(int i = 0; i<84; i++){
            for (Node n : graph){
                if(n.hasAttribute("infecte")){
                    for(Edge e : n){
                        nbInfect = estInfecte(e.getOpposite(n), nbInfect);
                    }
                }
                nbInfect = antiVerus(n, nbInfect);
            }
            System.out.println("Jour "+ i + " nombre infecte : " + nbInfect );
            //System.out.println(i +" "+ nbInfect);
        }
    }
    public static void propagationScenario2(Graph graph){
        int nbImmunise = 0;
        for(Node n : graph) {
            if ((int)(Math.random() * 2 + 1) == 1) {
                n.setAttribute("immunise", true);
                nbImmunise += 1;
            }
        }
        Node s = graph.getNode(0);
        s.setAttribute("infecte", true);
        int nbInfect = 1;
        for(int i = 0; i<84; i++){
            for (Node n : graph){
                if(n.hasAttribute("infecte")){
                    for(Edge e : n){
                        nbInfect = estInfecte(e.getOpposite(n), nbInfect);
                    }
                }
                nbInfect = antiVerus(n, nbInfect);
            }
            System.out.println("Jour "+ i + " nombre infecte : " + nbInfect + " Nombre d'immunise : " + nbImmunise );
            //System.out.println(i +" "+ nbInfect);

        }
        double moyenneDegre = 0.0;
        double moyenneCarreDegre;
        int somme=0;
        for(Node n : graph) {
            if(!n.hasAttribute("immunise")) {
                moyenneDegre += n.getDegree();
                somme += Math.pow(n.getDegree(), 2);
            }
        }
        // Nombre de noeuds à prendre en compte dans le graphe modifié
        int nbNoeuds = graph.getNodeCount() - nbImmunise;

        moyenneDegre /= nbNoeuds;
        moyenneCarreDegre = (double)(somme / nbNoeuds);


        System.out.println("Simulation 2 \n");
        System.out.println("Nombre de noeuds non immunisés : " + nbNoeuds);
        System.out.println("<k> : " + moyenneDegre);
        System.out.println("<k²> : " + moyenneCarreDegre);
        System.out.println("Seuil épidémique du réseau modifié <k> / <k²> : " + (moyenneDegre / moyenneCarreDegre));
    }

    public static void propagationScenario3(Graph graph){
        int nbImmunise = 0;
        double somme_degres_G0 = 0;
        double somme_degres_G1 = 0;
        for(Node n : graph) {
            if ((int)(Math.random() * 2 + 1) == 1 && n.edges().count() != 0) {
                int voisinAleat = (int)(Math.random() * n.edges().count() );
                n.getEdge(voisinAleat).getOpposite(n).setAttribute("immunise", true);
                nbImmunise += 1;
                somme_degres_G0 += n.getDegree();
                somme_degres_G1 += n.getEdge(voisinAleat).getOpposite(n).getDegree();
            }
        }
        Node s = graph.getNode(0);
        s.setAttribute("infecte", true);
        int nbInfect = 1;
        for(int i = 0; i<84; i++){
            for (Node n : graph){
                if(n.hasAttribute("infecte")){
                    for(Edge e : n){
                        nbInfect = estInfecte(e.getOpposite(n), nbInfect);
                    }
                }
                nbInfect = antiVerus(n, nbInfect);
            }
            System.out.println("Jour "+ i + " nombre infecte : " + nbInfect + " Nombre d'immunise : " + nbImmunise  );
            //System.out.println(i +" "+ nbInfect);
        }

        double moyenneDegre = 0.0;
        double moyenneCarreDegre;
        int somme=0;
        for(Node n : graph) {
            if(!n.hasAttribute("immunise")) {
                moyenneDegre += n.getDegree();
                somme += Math.pow(n.getDegree(), 2);
            }
        }
        // Nombre de noeuds à prendre en compte dans le graphe modifié
        int nbNoeuds = graph.getNodeCount() - nbImmunise;

        moyenneDegre /= nbNoeuds;
        moyenneCarreDegre = (double)(somme / nbNoeuds);

        double degre_moyen_G0 = somme_degres_G0/nbImmunise;
        double degre_moyen_G1 = somme_degres_G1/nbImmunise;

        System.out.println("Le degrés moyen du graphe 0 : " + degre_moyen_G0);
        System.out.println("Le degrés moyen du graphe 1 : " + degre_moyen_G1);

        System.out.println("Simulation 3 \n");
        System.out.println("Nombre de noeuds non immunisés : " + nbNoeuds);
        System.out.println("<k> : " + moyenneDegre);
        System.out.println("<k²> : " + moyenneCarreDegre);
        System.out.println("Seuil épidémique du réseau modifié <k> / <k²> : " + (moyenneDegre / moyenneCarreDegre));
    }



    public static void main(String[] args){
        String filePath = "com-dblp.ungraph.txt";
        Graph g = new DefaultGraph("g");
        FileSource fs = new FileSourceEdge();
        fs.addSink(g);
        System.setProperty("org.graphstream.ui", "swing");
        try {
            fs.readAll(filePath);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            fs.removeSink(g);
        }
        int nbNoeuds = g.getNodeCount();
        System.out.println("le nombre de noeuds du graphe est : "+ nbNoeuds);
        int nbEdges = g.getEdgeCount();
        System.out.println("le nombre de liens du graphe est : "+ nbEdges);
        double degreMoyen = averageDegree(g);
        System.out.println("le degre moyen du graphe est : "+ degreMoyen);
        double B = 0.1428;
        System.out.println("la probabilité de trassmission : "+ B);
        double U = 0.0714;
        System.out.println("la probabilité d'etre remis : "+ U);
        double landa = B/U;
        System.out.println("le taux de propagation du virus : "+ landa);
        double seuil = seuilEpidemique(g);
        double seuilAleatoire = seuilEpidemiqueReseauAleatoire(degreMoyen, nbNoeuds);
        if(seuil > seuilAleatoire){
            System.out.println("le seuil epidemique de notre reseau est plus grand que le seuil epidemique du reseau aleatoire");
        }else if(seuil < seuilAleatoire){
            System.out.println("le seuil epidemique de notre reseau est plus petit que le seuil epidemique du reseau aleatoire");
        }else{
            System.out.println("le seuil epidemique de notre reseau et le seuil epidemique du reseau aleatoire sont equivalents");
        }
        //propagationScenario1(g);
        //propagationScenario2(g);
        propagationScenario3(g);

    }
}
